/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>

// define some constants to make referencing in/outs clearer
#define PN_INPUT1 0
#define PN_INPUT2 1
#define PN_INPUT3 2
#define PN_INPUT4 3
#define PN_INPUT5 4
#define PN_OUTPUT1 5

// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *in1  = buffer_offset + input1_buffer;
    float *in2  = buffer_offset + input2_buffer;
    float *in3  = buffer_offset + input3_buffer;
	float *in4  = buffer_offset + input4_buffer;
	float *in5  = buffer_offset + input5_buffer;
	float *out1 = buffer_offset + output1_buffer;

    while(--sampleFrames >= 0) // sampleFrames = how many samples to process (can vary). repeat (loop) that many times
    {
		float control = *in1;	// get the sample 'POINTED TO' by in1
		float input2 = *in2;
		float input3 = *in3;
		float input4 = *in4;
		float input5 = *in5;

		// do the actual processing 
		
		if ( control < -0.00001f )
			{
			*out1 = input5;
			}
		else if ( (control > -0.00001f) && (control < 1.00001f) )
			{
			
			if ( control < 0.33333333333f)
				{
				float xvalue = (control/0.33333333333f);
				*out1 = (xvalue *(input4-input5)/1)+input5;
				}
			else if ( control < 0.6666666666)
				{
				float xvalue = ((control - 0.33333333333f) / 0.33333333333f);
				*out1 = (xvalue *(input3-input4)/1)+input4;
				}
			else if ( control > 0.6666666666f)
				{
				float xvalue = ((control - 0.6666666666f) / 0.33333333333f);
				*out1 = (xvalue *(input2-input3)/1)+input3;
				}
			}
		else if ( control > 1.00001f )
			{
			*out1 = input2;
			}

		// increment the pointers (move to next sample in buffers)
		in1++;
		in2++;
		in3++;
		in4++;
		in5++;
		out1++;
    }
}

void Module::sub_process_static(long buffer_offset, long sampleFrames )
{
	sub_process(buffer_offset, sampleFrames);
	static_count = static_count - sampleFrames;
	if( static_count <= 0 /* && !AreEventsPending()*/ )
	{
		CallHost(seaudioMasterSleepMode);
	}
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	// call the base class
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);

	// How to prevent other people using your custom module.
	// query the current registered user's name
	char host_registered_to[100];
	CallHost( seaudioMasterGetRegisteredName, 0,0, host_registered_to );
	bool registered_to_me = strcmp(host_registered_to, "Jeff McClintock") == 0;

	if( !registered_to_me )
	{
		// do something appropriate here if you don't want other people using your module
	}
}

Module::~Module()
{
	// This is where you free any memory/resources your module has created
}

// describe your module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "OL 2BPShaper";								// !!TODO!!

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "OL_2BPShaper";						// !!TODO!!

	// Info, may include Author, Web page whatever
	properties->about = "Shaper with 2 breakpoints\nversion 1.0\nby Oli Larkin\n\nwww.oli.adbe.org";								// !!TODO!!
	return true;
}

// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
		// typical input plug (inputs are listed first)
	case 0:
		properties->name				= "Control 0-10v";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case 1:
		properties->name				= "10";
		properties->variable_address	= &input2_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "10";
		break;
	case 2:
		properties->name				= "6.6";
		properties->variable_address	= &input3_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "6.6666";
		break;
	case 3:
		properties->name				= "3.3";
		properties->variable_address	= &input4_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "3.3333";
		break;
	case 4:
		properties->name				= "0";
		properties->variable_address	= &input5_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
		// typical output plug
	case 5:
		properties->name				= "Output";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}




// An input plug has changed value
void Module::OnPlugStateChange(SEPin *pin)
{
//	float test = getPin(PN_INPUT2)->getValue();
//	int test2 = SampleClock();

	// query the 'state of the input plugs...
	//     ST_RUN     = Normal Streaming Audio        (e.g. from an oscillator)
	//     ST_STATIC  = Fixed, unchanging input value (e.g. a slider at rest)
	state_type in_stat1 = getPin(PN_INPUT1)->getStatus();
	state_type in_stat2 = getPin(PN_INPUT2)->getStatus();
	state_type in_stat3 = getPin(PN_INPUT3)->getStatus();
	state_type in_stat4 = getPin(PN_INPUT4)->getStatus();
	state_type in_stat5 = getPin(PN_INPUT5)->getStatus();

	// we need to pass on the state of this module's output signal
	// it depends on the inputs...
	state_type out_stat = in_stat1;
	if( in_stat2 > out_stat)
		out_stat = in_stat2;
	else if( in_stat3 > out_stat)
		out_stat = in_stat3;
	else if( in_stat4 > out_stat)
		out_stat = in_stat4;
	else if( in_stat5 > out_stat)
		out_stat = in_stat5;
	// 'transmit' this modules new output status to next module 'downstream'
	getPin(PN_OUTPUT1)->TransmitStatusChange( SampleClock(), out_stat);

	// setup up 'sleep mode' or not
	if( out_stat < ST_RUN )
	{
		static_count = getBlockSize();
		SET_PROCESS_FUNC(Module::sub_process_static);
	}
	else
	{
		SET_PROCESS_FUNC(Module::sub_process);
	}
}
